@@ -1,7 +1,7 @@
 package cmd
 
 // DCSO FEVER
-// Copyright (c) 2017, 2018, DCSO GmbH
+// Copyright (c) 2017, 2018, 2019, DCSO GmbH
 
 import (
 	"io"
@@ -83,12 +83,12 @@ func mainfunc(cmd *cobra.Command, args []string) {
 			metricsSubmissionExchange := viper.GetString("metrics.submission-exchange")
 			statssubmitter, err = util.MakeAMQPSubmitterWithReconnector(metricsSubmissionURL,
 				metricsSubmissionExchange,
-				verbose, func(amqpURI string) (wabbit.Conn, string, error) {
+				verbose, func(amqpURI string) (wabbit.Conn, error) {
 					conn, err := amqp.Dial(amqpURI)
 					if err != nil {
-						return nil, "topic", err
+						return nil, err
 					}
-					return conn, "topic", err
+					return conn, err
 				})
 			if err != nil {
 				log.Fatal(err)
@@ -179,6 +179,15 @@ func mainfunc(cmd *cobra.Command, args []string) {
 		if pse != nil {
 			forwardHandler.(*processing.ForwardHandler).SubmitStats(pse)
 		}
+		rdns := viper.GetBool("active.rdns")
+		if rdns {
+			expiryPeriod := viper.GetDuration("active.rdns-cache-expiry")
+			forwardHandler.(*processing.ForwardHandler).EnableRDNS(expiryPeriod)
+			privateOnly := viper.GetBool("active.rdns-private-only")
+			if privateOnly {
+				forwardHandler.(*processing.ForwardHandler).RDNSHandler.EnableOnlyPrivateIPRanges()
+			}
+		}
 		forwardHandler.(*processing.ForwardHandler).Run()
 		defer func() {
 			c := make(chan bool)
@@ -195,9 +204,12 @@ func mainfunc(cmd *cobra.Command, args []string) {
 	bloomFilePath := viper.GetString("bloom.file")
 	bloomAlertPrefix := viper.GetString("bloom.alert-prefix")
 	bloomCompressed := viper.GetBool("bloom.zipped")
+	bloomBlacklist := viper.GetStringSlice("bloom.blacklist-iocs")
 	var bloomHandler *processing.BloomHandler
 	if bloomFilePath != "" {
-		bloomHandler, err = processing.MakeBloomHandlerFromFile(bloomFilePath, bloomCompressed, eventChan, forwardHandler, bloomAlertPrefix)
+		bloomHandler, err = processing.MakeBloomHandlerFromFile(bloomFilePath,
+			bloomCompressed, eventChan, forwardHandler, bloomAlertPrefix,
+			bloomBlacklist)
 		if err != nil {
 			log.Fatal(err)
 		}
@@ -547,6 +559,8 @@ func init() {
 	viper.BindPFlag("bloom.zipped", runCmd.PersistentFlags().Lookup("bloom-zipped"))
 	runCmd.PersistentFlags().StringP("bloom-alert-prefix", "", "BLF", "String prefix for Bloom filter alerts")
 	viper.BindPFlag("bloom.alert-prefix", runCmd.PersistentFlags().Lookup("bloom-alert-prefix"))
+	runCmd.PersistentFlags().StringSliceP("bloom-blacklist-iocs", "", []string{"/", "/index.htm", "/index.html"}, "Blacklisted strings in Bloom filter (will cause filter to be rejected)")
+	viper.BindPFlag("bloom.blacklist-iocs", runCmd.PersistentFlags().Lookup("bloom-blacklist-iocs"))
 
 	// IP blacklist alerting options
 	runCmd.PersistentFlags().StringP("ip-blacklist", "", "", "List with IP ranges to alert on")
@@ -564,6 +578,14 @@ func init() {
 	runCmd.PersistentFlags().StringP("flowextract-submission-exchange", "", "flows", "Exchange to which raw flow events will be submitted")
 	viper.BindPFlag("flowextract.submission-exchange", runCmd.PersistentFlags().Lookup("flowextract-submission-exchange"))
 
+	// Active enrichment options
+	runCmd.PersistentFlags().BoolP("active-rdns", "", false, "enable active rDNS enrichment for src/dst IPs")
+	viper.BindPFlag("active.rdns", runCmd.PersistentFlags().Lookup("active-rdns"))
+	runCmd.PersistentFlags().DurationP("active-rdns-cache-expiry", "", 2*time.Minute, "cache expiry interval for rDNS lookups")
+	viper.BindPFlag("active.rdns-cache-expiry", runCmd.PersistentFlags().Lookup("active-rdns-cache-expiry"))
+	runCmd.PersistentFlags().BoolP("active-rdns-private-only", "", false, "only do active rDNS enrichment for RFC1918 IPs")
+	viper.BindPFlag("active.rdns-private-only", runCmd.PersistentFlags().Lookup("active-rdns-private-only"))
+
 	// Logging options
 	runCmd.PersistentFlags().StringP("logfile", "", "", "Path to log file")
 	viper.BindPFlag("logging.file", runCmd.PersistentFlags().Lookup("logfile"))
