@@ -4,6 +4,7 @@ package processing
 // Copyright (c) 2017, DCSO GmbH
 
 import (
+	"encoding/json"
 	"net"
 	"sync"
 	"time"
@@ -25,6 +26,8 @@ type ForwardHandlerPerfStats struct {
 // event types to be forwarded.
 type ForwardHandler struct {
 	Logger              *log.Entry
+	DoRDNS              bool
+	RDNSHandler         *RDNSHandler
 	ForwardEventChan    chan []byte
 	OutputSocket        string
 	OutputConn          net.Conn
@@ -177,8 +180,24 @@ func MakeForwardHandler(reconnectTimes int, outputSocket string) *ForwardHandler
 func (fh *ForwardHandler) Consume(e *types.Entry) error {
 	doForwardThis := util.ForwardAllEvents || util.AllowType(e.EventType)
 	if doForwardThis {
-		jsonCopy := make([]byte, len(e.JSONLine))
-		copy(jsonCopy, e.JSONLine)
+		var ev types.EveEvent
+		err := json.Unmarshal([]byte(e.JSONLine), &ev)
+		if err != nil {
+			return err
+		}
+		if fh.DoRDNS && fh.RDNSHandler != nil {
+			err = fh.RDNSHandler.Consume(e)
+			if err != nil {
+				return err
+			}
+			ev.SrcHost = e.SrcHosts
+			ev.DestHost = e.DestHosts
+		}
+		var jsonCopy []byte
+		jsonCopy, err = json.Marshal(ev)
+		if err != nil {
+			return err
+		}
 		fh.ForwardEventChan <- jsonCopy
 		fh.Lock.Lock()
 		fh.PerfStats.ForwardedPerSec++
@@ -201,6 +220,13 @@ func (fh *ForwardHandler) GetEventTypes() []string {
 	return util.GetAllowedTypes()
 }
 
+// EnableRDNS switches on reverse DNS enrichment for source and destination
+// IPs in outgoing EVE events.
+func (fh *ForwardHandler) EnableRDNS(expiryPeriod time.Duration) {
+	fh.DoRDNS = true
+	fh.RDNSHandler = MakeRDNSHandler(util.NewHostNamer(expiryPeriod, 2*expiryPeriod))
+}
+
 // Run starts forwarding of JSON representations of all consumed events
 func (fh *ForwardHandler) Run() {
 	if !fh.Running {
